#vider la mmoire
rm(list=ls())

#changer de rpertoire
setwd("D:/DataMining/Databases_for_mining/dataset_for_soft_dev_and_comparison/efficient_r_programming")

#charger les donnes
D <- read.csv("wave1000.txt",header=TRUE,sep="\t",dec=".")
print(str(D))

#library MASS
library(MASS)

#leave-one-out - version 1
lvo_v1 <- function()
{
  #vecteur de rsultat
  v <- c()
  #boucler
  for (i in 1:nrow(D)){
    #donnes de test
    DTest <- D[i,]
    #donnes d'apprentissage
    DTrain <- D[-i,]
    #modlisation
    modele <- lda(formula=onde ~ ., data=DTrain)
    #prdiction
    pred <- predict(modele, newdata=DTest)
    #rcupre la prdiction
    v <- c(v,pred$class[1])
  }
  #matrice de confusion
  mc <- table(D$onde,v)
  #taux d'erreur
  err <- 1.0-sum(diag(mc))/sum(mc)
}

#vrification
system.time(print(lvo_v1()))

#package pour mesurer le temps de calcul
library(microbenchmark)

#mesure avec 5 rptitions
print(microbenchmark(times=5,lvo_v1()))

#package pour profilage de code
library(profvis)

#profiler le code
profvis(
  {
    #vecteur de rsultat
    v <- c()
    #boucler
    for (i in 1:nrow(D)){
      #donnes de test
      DTest <- D[i,]
      #donnes d'apprentissage
      DTrain <- D[-i,]
      #modlisation
      modele <- lda(formula=onde ~ ., data=DTrain)
      #prdiction
      pred <- predict(modele, newdata=DTest)
      #rcupre la prdiction
      v <- c(v,pred$class[1])
    }
    #matrice de confusion
    mc <- table(D$onde,v)
    #taux d'erreur
    err <- 1.0-sum(diag(mc))/sum(mc)
  }  
)


#seconde version
#modifier l'appel de lda et predict
lvo_v2 <- function()
{
  #vecteur de rsultat
  v <- c()
  #boucler
  for (i in 1:nrow(D)){
    #donnes de test
    DTest <- D[i,]
    #donnes d'apprentissage
    DTrain <- D[-i,]
    #modlisation
    modele <- lda(x=DTrain[,-1],grouping=DTrain$onde)
    #prdiction
    pred <- predict(modele, newdata=DTest[,-1])
    #rcupre la prdiction
    v <- c(v,pred$class[1])
  }
  #matrice de confusion
  mc <- table(D$onde,v)
  #taux d'erreur
  err <- 1.0-sum(diag(mc))/sum(mc)
} 

#vrification
system.time(print(lvo_v2()))

#benchmarking
print(microbenchmark(times=5,lvo_v1(),lvo_v2()))

#profiler le code - version 2
profvis(
  {
    #vecteur de rsultat
    v <- c()
    #boucler
    for (i in 1:nrow(D)){
      #donnes de test
      DTest <- D[i,]
      #donnes d'apprentissage
      DTrain <- D[-i,]
      #modlisation
      modele <- lda(x=DTrain[,-1],grouping=DTrain$onde)
      #prdiction
      pred <- predict(modele, newdata=DTest[,-1])
      #rcupre la prdiction
      v <- c(v,pred$class[1])
    }
    #matrice de confusion
    mc <- table(D$onde,v)
    #taux d'erreur
    err <- 1.0-sum(diag(mc))/sum(mc)
  }  
)


#troisme version - passer par des matrices et vecteurs
#structures plus efficaces
lvo_v3 <- function()
{
  #matrice des descripteurs
  X <- as.matrix(D[,-1])
  y <- D[,1]
  #vecteur de rsultat
  v <- c()
  #boucler
  for (i in 1:nrow(D)){
    #donnes de test
    XTest <- X[i,]
    #donnes d'apprentissage
    XTrain <- X[-i,]
    yTrain <- y[-i]
    #modlisation
    modele <- lda(x=XTrain,grouping=yTrain)
    #prdiction
    pred <- predict(modele, newdata=XTest)
    #rcupre la prdiction
    v <- c(v,pred$class[1])
  }
  #matrice de confusion
  mc <- table(D$onde,v)
  #taux d'erreur
  err <- 1.0-sum(diag(mc))/sum(mc)
} 

#vrification
system.time(print(lvo_v3()))

#benchmarking
print(microbenchmark(times=5,lvo_v2(),lvo_v3()))

#analyse du code
profvis(
  {
    #matrice des descripteurs
    X <- as.matrix(D[,-1])
    y <- D[,1]
    #vecteur de rsultat
    v <- c()
    #boucler
    for (i in 1:nrow(D)){
      #donnes de test
      XTest <- X[i,]
      #donnes d'apprentissage
      XTrain <- X[-i,]
      yTrain <- y[-i]
      #modlisation
      modele <- lda(x=XTrain,grouping=yTrain)
      #prdiction
      pred <- predict(modele, newdata=XTest)
      #rcupre la prdiction
      v <- c(v,pred$class[1])
    }
    #matrice de confusion
    mc <- table(D$onde,v)
    #taux d'erreur
    err <- 1.0-sum(diag(mc))/sum(mc)
  }
)

#librairie pour prog. parallle
library(parallel)

#programmation parallle de la LOOCV
lvo_parallel <- function(){
  #fonction comptage des erreurs
  chunk_lvo <- function(index,descriptors,y){
    v <- c()
    for (i in index){
      #donnes de test
      XTest <- descriptors[i,]
      #donnes d'apprentissage
      XTrain <- descriptors[-i,]
      yTrain <- y[-i]
      #modlisation
      modele <- MASS::lda(x=XTrain,grouping=yTrain)
      #prdiction
      pred <- predict(modele, newdata=XTest)
      #rcupre la prdiction
      v <- c(v,pred$class)
    }
    #dcompte des erreur
    nb_errs <- sum(as.numeric(unclass(y[index]) != v))
    return(nb_errs)
  }
  #matrice des descripteurs
  XD <- D[,-1]
  yD <- D[,1]
  #spliter le vecteur dsignant les individus en 4 listes
  id <- 1:nrow(D)
  lst_id <- split(id,id%/%(nrow(D)%/%4+1))
  #prparer les 4 clusters
  cl = makeCluster(4)
  #stopper les culsters  la sortie de la fonction
  on.exit(stopCluster(cl))  
  #lancer les calculs - on a vecteurs des erreurs par chuk
  erreurs <- parSapply(cl,X=lst_id,chunk_lvo,descriptors=XD,y=yD)
  #taux d'erreur
  return(sum(erreurs)/nrow(D))
}

#appel de la fonction
system.time(print(lvo_parallel()))

#comparaisons
print(microbenchmark(times=5,lvo_v3(),lvo_parallel()))

#utiliser l'implmentation native de lda
lvo_lda <- function()
{
  #matrice des descripteurs
  X <- as.matrix(D[,-1])
  y <- D[,1]
  #lda avec LVO
  p <- lda(x=X,grouping=y,CV=TRUE)
  #matrice de confusion
  mc <- table(D$onde,p$class)
  #taux d'erreur
  err <- 1.0-sum(diag(mc))/sum(mc)
} 

#vrification
system.time(print(lvo_lda()))

#***********************************************
#pourquoi les rsultats ne sont pas identiques ?

#distribution des classes sur la totalit des donnes
proba <- as.numeric(table(D$onde)/nrow(D))
print(proba)

#programmation du LVO
#pour retrouver les rsultats de lda(CV=TRUE)
lvo_my_lda <- function()
{
  #matrice des descripteurs
  X <- as.matrix(D[,-1])
  y <- D[,1]
  #vecteur de rsultat
  v <- c()
  #boucler
  for (i in 1:nrow(D)){
    #donnes de test
    XTest <- X[i,]
    #donnes d'apprentissage
    XTrain <- X[-i,]
    yTrain <- y[-i]
    #modlisation
    modele <- lda(x=XTrain,grouping=yTrain,prior=proba)
    #prdiction
    pred <- predict(modele, newdata=XTest)
    #rcupre la prdiction
    v <- c(v,pred$class[1])
  }
  #matrice de confusion
  mc <- table(D$onde,v)
  #taux d'erreur
  err <- 1.0-sum(diag(mc))/sum(mc)
} 

#vrification
print(lvo_my_lda())

